#!/usr/bin/python
from pprint import pprint
from copy import copy
from random import randint

## classes

class World:
	def __init__(self, constants = {}):
		self.n = 10

		self.init_stock_limit = 2

		self.bonus_limit = 2
		self.effort_limit = 10
		self.upkeep = 6
		self.combiner = min

		self.__dict__.update(constants)
		
		self.locs = range(self.n)
		self.bonus = [randint(0,self.bonus_limit) for x in self.locs]

	def get_initial_state(self):
		state = State(self)
		state.initialize_stock()
		return state

	def pre_move(self, state):
		state.compute_effort()

	def apply_move(self, state, move):
		move(state)

	def post_move(self, state):
		state.compute_stock()
		state.compute_output()
		state.compute_happiness()


class State:
	def __init__(self, world):
		self.world = world
		self.happiness = 0
		self.stock = [0 for x in self.world.locs]
		self.effort = [0 for x in self.world.locs]
		self.output = [0 for x in self.world.locs]

	def initialize_stock(self):
		self.stock = [randint(0,self.world.init_stock_limit) for x in self.world.locs]

	def compute_effort(self):
		self.effort = [self.world.bonus[x] + randint(0,self.world.effort_limit) for x in self.world.locs]

	def compute_output(self):
		comb = self.world.combiner
		n = self.world.n
		def output_for(x):
			if x is 0:
				return comb([self.stock[-1], self.stock[0], self.stock[1]])
			elif x is n-1:
				return comb([self.stock[0], self.stock[n-2], self.stock[n-1]])
			else:
				return comb([self.stock[x-1], self.stock[x], self.stock[x+1]])

		self.output = [output_for(x) for x in self.world.locs]

	def compute_stock(self):
		def stock_for(x):
			return max(0, self.stock[x] + self.effort[x] - self.world.upkeep)

		self.stock = [stock_for(x) for x in self.world.locs]

	def compute_happiness(self):
		self.happiness = sum(self.output)

	def dump(self):
		for k in ['effort','stock','output','happiness']:
			print "\t%s: %s" % (k, self.__dict__[k])

## drivers

def main(argv):
	simulate(id,10)

def simulate(move,t_max):
	world = World()
	state = world.get_initial_state()
	for t in range(t_max):
		world.pre_move(state)
		world.apply_move(state, move)
		world.post_move(state)
		print "turn %d:" % t
		state.dump()
		print "\n"

## top level
if __name__ == '__main__':
	import sys
	main(sys.argv)

